/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	JavaScript functions for the bioplates and well mapping

	@author Nicklas
	@version 2.16
*/

function Plate(plateNo, prefix, name)
{
	this.plateNo = plateNo;
	this.prefix = prefix;
	this.name = name;
	this.description = '';
	this.barcode = '';
	this.wells = new Array();
	
	Plate.prototype.addSourceWell = function(row, column)
	{
		var tag = document.getElementById(this.prefix + '.' + row + '.' + column);
		var well = new Well(this, row, column, tag);
		Main.addClass(tag, 'filled editable unmapped');
		this.wells[this.wells.length] = well;
		this.wells[row + ':' + column] = well;
		return well;
	}
	
	Plate.prototype.addDestWell = function(row, column, srcPlate, srcRow, srcCol)
	{
		var tag = document.getElementById(this.prefix + '.' + row + '.' + column);
		var well = new Well(this, row, column, tag);
		if (srcPlate)
		{
			var srcWell = srcPlate.getWell(srcRow, srcCol);
			if (srcWell) srcWell.mapToDestination(well, 1);
		}
		this.wells[this.wells.length] = well;
		this.wells[row + ':' + column] = well;
		return well;
	}

	Plate.prototype.getWell = function(row, column)
	{
		return this.wells[row + ':' + column];
	}

	Plate.prototype.mapToDestination = function(row, column, destPlate, destRow, destCol)
	{
		if (!destPlate) return;
		var srcWell = this.getWell(row, column);
		if (!srcWell) return;
		
		var destWell = destPlate.getWell(destRow, destCol);
		if (!destWell) return;
		srcWell.mapToDestination(destWell, 0);
	}
	
	Plate.prototype.clearAllMappings = function()
	{
		for (var i = 0; i < this.wells.length; i++)
		{
			this.wells[i].clearMapping();
		}
	}
	
	Plate.prototype.getNumMappedWells = function()
	{
		var count = 0;
		for (var i = 0; i < this.wells.length; i++)
		{
			var well = this.wells[i];
			if (well.srcWell || well.destWells.length > 0) count++;
		}
		return count;
	}
	
	Plate.prototype.toForm = function(frm)
	{
		var prefix = 'plate.' + this.plateNo + '.';
		if (this.name) Forms.createHidden(frm, prefix+'name', this.name);
		if (this.description) Forms.createHidden(frm, prefix+'description', this.description);
		if (this.barcode) Forms.createHidden(frm, prefix+'barcode', this.barcode);
		
		for (var i = 0; i < this.wells.length; i++)
		{
			var well = this.wells[i];
			if (well.srcWell)
			{
				well.toForm(frm);
			}
		}
	}
	
}

var toAlpha = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O'];
function Well(plate, row, column, tag)
{
	this.plate = plate;
	this.row = row;
	this.column = column;
	this.tag = tag;
	this.srcWell = null;
	this.name = '';
	this.childName = '';
	this.childDescription = '';
	this.destWells = new Array();
	
	Well.prototype.getCoordinate = function()
	{
		return toAlpha[this.row] + (this.column+1);
	}
	
	Well.prototype.mapToDestination = function(destWell, keepExisting)
	{
		if (!keepExisting) this.clearMapping();
		destWell.unlinkSource();
		destWell.srcWell = this;
		Main.addClass(destWell.tag, 'mapped');
		this.destWells[this.destWells.length] = destWell;
		Main.removeClass(this.tag, 'unmapped');
	}

	Well.prototype.clearMapping = function()
	{
		for (var i = 0; i < this.destWells.length; ++i)
		{
			var oldDestWell = this.destWells[i];
			Main.removeClass(oldDestWell.tag, 'mapped');
			Main.removeClass(oldDestWell.tag, 'mapref');
			oldDestWell.srcWell = null;
		}
		this.destWells = new Array();
		Main.addClass(this.tag, 'unmapped');
	}
	
	Well.prototype.unlinkSource = function()
	{
		if (!this.srcWell) return;
		
		var srcWell = this.srcWell;
		for (var i = 0; i < srcWell.destWells.length; ++i)
		{
			if (srcWell.destWells[i] == this)
			{
				srcWell.destWells.splice(i, 1);
				break;
			}
		}
		if (srcWell.destWells.length == 0)
		{
			Main.addClass(srcWell.tag, 'unmapped');
			Main.removeClass(srcWell.tag, 'mapref');
		}
		this.srcWell = null;
		Main.removeClass(this.tag, 'mapped');
	}
	
	Well.prototype.showMapping = function(graphics, pen)
	{
		if (this.srcWell)
		{
			var srcPos = Main.getElementPosition(this.srcWell.tag);
			var destPos = Main.getElementPosition(this.tag);
			var points = new Array();
			points[0] = new jsPoint(srcPos.left + srcPos.width/2, srcPos.top+srcPos.height/2);
			points[1] = new jsPoint(destPos.left+destPos.width/2, destPos.top+destPos.height/2);
			graphics.drawLine(pen, points[0], points[1]);
			Main.addClass(this.srcWell.tag, 'mapref');
		}
		else if (this.destWells.length > 0)
		{
			for (var i = 0; i < this.destWells.length; i++)
			{
				this.destWells[i].showMapping(graphics, pen);
				Main.addClass(this.destWells[i].tag, 'mapref');
			}
		}
	}
	
	Well.prototype.hideMapping = function(graphics)
	{
		if (this.srcWell)
		{
			graphics.clear();
			Main.removeClass(this.srcWell.tag, 'mapref');
		}
		else if (this.destWells.length > 0)
		{
			for (var i = 0; i < this.destWells.length; i++)
			{
				this.destWells[i].hideMapping(graphics);
				Main.removeClass(this.destWells[i].tag, 'mapref');
			}
		}
	}
	
	Well.prototype.toForm = function(frm)
	{
		if (!this.srcWell) return;
		var prefix = 'well.' + this.plate.plateNo + '.' + this.row + '.' + this.column + '.';
		Forms.createHidden(frm, prefix + 'source', this.srcWell.row + ',' + this.srcWell.column);
		var name = this.srcWell.childName;
		if (name) Forms.createHidden(frm, prefix+'name', name);
		var description = this.srcWell.childDescription;
		if (description) Forms.createHidden(frm, prefix+'description', description);
	}

}

