/* $Id: tabcontrol.js 5091 2009-08-31 09:21:04Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	JavaScript functions for the TabControl taglib.

	@author Nicklas
	@version 2.0
*/

var TabControl = new TabControlClass();
function TabControlClass()
{
	this.autoAdjust = new Array();

	/**
		Set the initial active tab. It the checkRememebered
		parameter is set it will look check the top or opener window for
		a remembered active tab.
	*/
	this.setInitialTab = function(tabControlId, tabId, checkRemembered, switchFunction, autoAdjustHeight)
	{
		if (checkRemembered)
		{
			var tabControl = document.getElementById(tabControlId);
			if (!tabControl.activeTab)
			{
				var theWindow = window.opener ? window.opener : window;
				var rememberedTabId = TabControl.getRememberedTabId(theWindow, tabControlId);
				if (rememberedTabId) tabId = rememberedTabId
			}
		}
		if (switchFunction)
		{
			switchFunction(tabControlId, tabId);
		}
		else
		{
			TabControl.setActiveTab(tabControlId, tabId);
		}
		
		// Set initial size of tab control as compared to the window
		if (autoAdjustHeight)
		{
			var tabContent = document.getElementById(tabControlId+'.content');
			var tabHeight = parseInt(tabContent.style.height);
			var windowHeight = Main.getWindowHeight();
			if (tabHeight && windowHeight)
			{
				TabControl.autoAdjust[TabControl.autoAdjust.length] = tabControlId;
				tabContent.initialHeightDifference = windowHeight - tabHeight;
			}
		}
	}

	/*
		Switch to another tab.
		@param tabControlId The ID attribute of the main tabcontrol
		@param tabId The ID attribute of the tab to make active
		@return If the call resulted in a switch
	*/
	this.setActiveTab = function(tabControlId, tabId)
	{
		var tabControl = document.getElementById(tabControlId);
		var theWindow = window.opener ? window.opener : window;
	
		var realTabId = tabControlId+'.'+tabId;
		var tab = document.getElementById(realTabId);
		if (tabControl.activeTab == tab) return false;
		if (!TabControl.validateActiveTab(tabControlId)) return false;
	
		if (tabControl.activeTab != null)
		{
			if (tabControl.className == 'tabcontrol_bottom')
			{
				tabControl.activeTab.className = 'tab_bottom';
			}
			else
			{
				tabControl.activeTab.className = 'tab';
			}
		}
		if (tabControl.className == 'tabcontrol_bottom')
		{
			tab.className = 'activetab_bottom';
		}
		else
		{
			tab.className = 'activetab';
		}
		tabControl.activeTab = tab;
		if (tabControl.activeContent != null)
		{
			tabControl.activeContent.style.display = 'none';
		}
		var content = document.getElementById(realTabId+'.content');
		if (content != null)
		{
			content.style.display = 'block';
			tabControl.activeContent = content;
		}
		TabControl.rememberActiveTab(theWindow, tabControlId, tabId);
		return true;
	}

	this.rememberActiveTab = function(window, tabControlId, tabId)
	{
		if (window.Main) window.Main.setPageValue(location.pathname, tabControlId, tabId);
	}
	
	this.getRememberedTabId = function(window, tabControlId)
	{
		if (!window.Main) return null;
		return window.Main.getPageValue(location.pathname, tabControlId);
	}
	
	this.getActiveTabId = function(tabControlId)
	{
		var tabControl = document.getElementById(tabControlId);
		var activeTab = tabControl.activeTab;
		if (activeTab)
		{
			return activeTab.id.substr(tabControlId.length + 1);
		}
	}
	
	this.isActive = function(tabControlId, tabId)
	{
		var realTabId = tabControlId+'.'+tabId;
		var tabControl = document.getElementById(tabControlId);
		var tab = document.getElementById(realTabId);
		return tabControl.activeTab == tab;
	}
	
	this.getActiveTabHelpId = function(tabControlId)
	{
		var tabControl = document.getElementById(tabControlId);
		var activeTab = tabControl.activeTab;
		if (activeTab)
		{
			return activeTab.helpId;
		}
	}

	this.mouseEnter = function(tabControlId, tabId)
	{
		var realTabId = tabControlId+'.'+tabId;
		var tabControl = document.getElementById(tabControlId);
		var tab = document.getElementById(realTabId);
		if (tabControl.activeTab == tab) return;
		if (tabControl.className == 'tabcontrol_bottom')
		{
			tab.className = 'tab_bottom_hover';
		}
		else
		{
			tab.className = 'tab_hover';
		}
	}

	this.mouseLeave = function(tabControlId, tabId)
	{
		var realTabId = tabControlId+'.'+tabId;
		var tabControl = document.getElementById(tabControlId);
		var tab = document.getElementById(realTabId);
		if (tabControl.activeTab == tab) return;
		if (tabControl.className == 'tabcontrol_bottom')
		{
			tab.className = 'tab_bottom';
		}
		else
		{
			tab.className = 'tab';
		}
	}

	/**
		Validates the active tab
		@param tabControlId The ID attribute of the main tabcontrol
	*/
	this.validateActiveTab = function(tabControlId)
	{
		var tabControl = document.getElementById(tabControlId);
		if (tabControl.activeTab != null && tabControl.activeTab.validate != null)
		{
			return eval(tabControl.activeTab.validate);
		}
		else
		{
			return true;
		}
	}

	/**
		Set the validation function and help id for a tab
		@param tabControlId The ID attribute of the main tabcontrol
		@param tabId The ID attribute of the tab
		@param validate The function that validates the tab
		@param helpId The help id for the tab
	*/
	this.initTab = function(tabControlId, tabId, validate, helpId)
	{
		var realTabId = tabControlId+'.'+tabId;
		var tab = document.getElementById(realTabId);
		if (tab != null) 
		{
			if (validate) tab.validate = validate;
			if (helpId) tab.helpId = helpId;
		}
	}
	
	this.adjustTabHeights = function()
	{
		for (var i = 0; i < TabControl.autoAdjust.length; i++)
		{
			var tabControlId = TabControl.autoAdjust[i];
			var tabContent = document.getElementById(tabControlId + '.content');
			if (tabContent)
			{
				var heightDiff = tabContent.initialHeightDifference;
				if (heightDiff != undefined)
				{
					var winHeight = Main.getWindowHeight();
					var newHeight = winHeight-heightDiff;
					if (newHeight >= 0) tabContent.style.height = newHeight+'px';
				}
			}
		}
	}
}

window.oldOnresize = window.onresize;
window.onresize = function()
{
	if (window.oldOnresize) window.oldOnresize();
	TabControl.adjustTabHeights();
}

